#include "stdafx.h"
#include "WinNotify.h"
#include "WinNotifyDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CWinNotifyDoc, CDocument)

BEGIN_MESSAGE_MAP(CWinNotifyDoc, CDocument)
	//{{AFX_MSG_MAP(CWinNotifyDoc)
	ON_COMMAND(ID_MESSAGE_CLEARALL, OnMessageClearAll)
	ON_UPDATE_COMMAND_UI(ID_MESSAGE_CLEARALL, OnUpdateMessageClearAll)
	ON_COMMAND(ID_MESSAGE_DISCARD, OnMessageDiscard)
	ON_UPDATE_COMMAND_UI(ID_MESSAGE_DISCARD, OnUpdateMessageDiscard)
	ON_COMMAND(ID_MESSAGE_NEXT, OnMessageNext)
	ON_UPDATE_COMMAND_UI(ID_MESSAGE_NEXT, OnUpdateMessageNext)
	ON_COMMAND(ID_MESSAGE_PREVIOUS, OnMessagePrevious)
	ON_UPDATE_COMMAND_UI(ID_MESSAGE_PREVIOUS, OnUpdateMessagePrevious)
	ON_COMMAND(ID_MESSAGE_SEND, OnMessageSend)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CWinNotifyDoc::CWinNotifyDoc()
{
  m_nCurrentMessageIndex = 0;
}

CWinNotifyDoc::~CWinNotifyDoc()
{
}

BOOL CWinNotifyDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

  //Create the server mailslot to listen for incoming messages
  if (!m_ServerMailslot.Open(_T("WinNot\\Server")))
  {
    AfxMessageBox(IDS_FAIL_CREATE_SERVER_MAILSLOT);
    return FALSE;
  }

	return TRUE;
}

int CWinNotifyDoc::GetCurrentMessageCount() const
{
  //return the current number of messages
	if (m_Messages.GetSize())
    return m_nCurrentMessageIndex+1;
  else
    return 0;
}

void CWinNotifyDoc::Serialize(CArchive& /*ar*/)
{
  //We do not do anything with the hard disk
}

#ifdef _DEBUG
void CWinNotifyDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CWinNotifyDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


void CWinNotifyDoc::OnMessageClearAll() 
{
  //remove all messages from the array 
  //and inform all views
  m_Messages.RemoveAt(0, m_Messages.GetSize());
  m_nCurrentMessageIndex = 0;
  UpdateAllViews(NULL);
}

void CWinNotifyDoc::OnUpdateMessageClearAll(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_Messages.GetSize() > 0);
}

void CWinNotifyDoc::OnMessageDiscard() 
{
  //remove the current message from the array 
  //and inform all views
	m_Messages.RemoveAt(m_nCurrentMessageIndex);
  if (m_nCurrentMessageIndex > m_Messages.GetUpperBound())
    m_nCurrentMessageIndex = m_Messages.GetUpperBound();
	UpdateAllViews(NULL);
}

void CWinNotifyDoc::OnUpdateMessageDiscard(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(m_Messages.GetSize() > 0);		
}

void CWinNotifyDoc::OnMessageNext() 
{
  //Move to the next message
  //and inform all views
	++m_nCurrentMessageIndex;
  UpdateAllViews(NULL);
}

void CWinNotifyDoc::OnUpdateMessageNext(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_nCurrentMessageIndex < m_Messages.GetUpperBound());
}

void CWinNotifyDoc::OnMessagePrevious() 
{
  //Move to the previous message
  //and inform all views
	--m_nCurrentMessageIndex;
  UpdateAllViews(NULL);
}

void CWinNotifyDoc::OnUpdateMessagePrevious(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_nCurrentMessageIndex > 0);
}

void CWinNotifyDoc::OnMessageSend() 
{
  //send a message by opening a client side mailslot
  //and writing to it
	CSendDlg dlg;
  if (dlg.DoModal() == IDOK)
  {
    CClientMailslot mailslot;
    if (!mailslot.Open(dlg.m_sTo, _T("WinNot\\Server")))
    {
      AfxMessageBox(IDS_FAIL_CREATE_CLIENT_MAILSLOT);
      return;
    }

    //create the message
    CNotifyMessage msg;
    _tcscpy(msg.m_sMessage, dlg.m_sMessage);

    TCHAR sUserName[MAX_NAME_SIZE];
    DWORD dwUserNameSize = MAX_NAME_SIZE;
    if (GetUserName(sUserName, &dwUserNameSize))
      _tcscpy(msg.m_sSenderName, sUserName);
    else
      _tcscpy(msg.m_sSenderName, _T("unknown"));

    TCHAR sComputerName[MAX_NAME_SIZE];
    DWORD dwComputerNameSize = MAX_NAME_SIZE;
    if (GetComputerName(sComputerName, &dwComputerNameSize))
      _tcscpy(msg.m_sSenderMachine, sComputerName);
    else
      _tcscpy(msg.m_sSenderMachine, _T("unknown"));

    _tcscpy(msg.m_sTo, dlg.m_sTo);
    GetLocalTime(&msg.m_SentTime);

    //Send the message
    BeginWaitCursor();
    DWORD dwBytesWritten = 0;
    DWORD dwBytesToWrite = sizeof(CNotifyMessage);
    if (!mailslot.Write(&msg, dwBytesToWrite, &dwBytesWritten) ||
        (dwBytesToWrite != dwBytesWritten))
    {
      EndWaitCursor();
      AfxMessageBox(IDS_FAIL_SEND_MESSAGE);
      return;
    }
    EndWaitCursor();
    AfxMessageBox(IDS_MESSAGE_SENT);
  }
}

void CWinNotifyDoc::CheckForNewMail()
{
  //Read all the messages waiting in 
  //the server mailslot
  while (m_ServerMailslot.MessageWaiting())
  {
    DWORD dwMsgSize = m_ServerMailslot.SizeOfWaitingMessage();
    BYTE* pMsg = new BYTE[dwMsgSize];
    
    DWORD dwNumberOfBytesRead=0;
    if (m_ServerMailslot.Read(pMsg, dwMsgSize, &dwNumberOfBytesRead))
    {
      if (dwNumberOfBytesRead == sizeof(CNotifyMessage))
      {
        CNotifyMessage msg;
        memcpy(&msg, pMsg, dwNumberOfBytesRead);
        m_Messages.Add(msg);
        m_nCurrentMessageIndex = m_Messages.GetUpperBound();
        UpdateAllViews(NULL);
        MessageBeep(MB_ICONEXCLAMATION);
      }
    }
    delete []pMsg;
  }
}

CString CWinNotifyDoc::GetCurrentMessage() const 
{
  //Retreives the text for the current message
  CString rVal;

  if (m_Messages.GetSize())
    rVal = m_Messages.GetAt(m_nCurrentMessageIndex).m_sMessage;
  
  return rVal;
}


CString CWinNotifyDoc::GetCurrentSender() const
{
  //Retreives the sender for the current message
  CString rVal;

  if (m_Messages.GetSize())
    rVal = m_Messages.GetAt(m_nCurrentMessageIndex).m_sSenderName;

  return rVal;
}

CString CWinNotifyDoc::GetCurrentSenderMachine() const
{
  //Retreives the senders machine for the current message
  CString rVal;

  if (m_Messages.GetSize())
    rVal = m_Messages.GetAt(m_nCurrentMessageIndex).m_sSenderMachine;

  return rVal;
}

CString CWinNotifyDoc::GetCurrentTo() const
{
  //Retreives the receiver for the current message
  CString rVal;

  if (m_Messages.GetSize())
    rVal = m_Messages.GetAt(m_nCurrentMessageIndex).m_sTo;
  
  return rVal;
}


CString CWinNotifyDoc::GetCurrentMessageTime() const
{
  //Retreives the time stamp for the current message
  CString rVal;

  if (m_Messages.GetSize())
  {
    TCHAR pszDateStr[20];
    TCHAR pszTimeStr[20];
    GetDateFormat(LOCALE_USER_DEFAULT, LOCALE_NOUSEROVERRIDE, &m_Messages.GetAt(m_nCurrentMessageIndex).m_SentTime, NULL, pszDateStr, 20);
    GetTimeFormat(LOCALE_USER_DEFAULT, LOCALE_NOUSEROVERRIDE, &m_Messages.GetAt(m_nCurrentMessageIndex).m_SentTime, NULL, pszTimeStr, 20);
    rVal = CString(pszDateStr) + CString(_T(" ")) + pszTimeStr;
  }
  
  return rVal;
}



CSendDlg::CSendDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSendDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSendDlg)
	m_sMessage = _T("");
	m_sTo = _T("");
	//}}AFX_DATA_INIT
}


void CSendDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSendDlg)
	DDX_Text(pDX, IDC_TO, m_sTo);
	DDX_Text(pDX, IDC_MESSAGE, m_sMessage);
	//}}AFX_DATA_MAP
  pDX->PrepareEditCtrl(IDC_TO);
  DDV_MaxChars(pDX, m_sTo, MAX_NAME_SIZE);
  pDX->PrepareEditCtrl(IDC_MESSAGE);
  DDV_MaxChars(pDX, m_sMessage, MAX_MESSAGE_SIZE);
}


BEGIN_MESSAGE_MAP(CSendDlg, CDialog)
	//{{AFX_MSG_MAP(CSendDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

